### Warning: some computations may require some time to complete ###

library("stepp")

# Section 3 - The stepp package
data("simdataKM", package = "stepp")

swin <- new("stwin", type = "sliding", r1 = 200, r2 = 300)
subp <- new("stsubpop")
subp <- generate(subp, win = swin, covariate = simdataKM$covar)
summary(subp)

res <- new("steppes")
modelKM <- new("stmodelKM", coltrt = simdataKM$trt,
  survTime = simdataKM$time, censor = simdataKM$censor,
  trts = c(1, 2), timePoint = 4)
res <- estimate(res, subp, modelKM)

set.seed(101)
nperm <- 500
res <- test(res, nperm)
# summary(res)
print(res, estimate = TRUE, cov = FALSE, test = TRUE)

set.seed(101)
res <- stepp.test(subp, modelKM, nperm)

plot(res, subplot = TRUE, ylabel = "Survival",
  tlegend = c("Treatment 1", "Treatment 2"),
  legend_diff = c(1, 2))

# Section 4 - Application
data("bigCI", package = "stepp")

swin <- new("stwin", type = "sliding_events", e1 = 5, e2 = 15)
subp <- new("stsubpop")
subp <- generate(subp, win = swin, covariate = bigCI$ki67,
  coltrt = bigCI$trt, trts = c(1, 2), coltype = bigCI$event)
summary(subp)

res <- new("steppes")
modelCI <- new("stmodelCI", coltrt = bigCI$trt, coltime = bigCI$time,
  coltype = bigCI$event, trts = c(1, 2), timePoint = 4)

res <- estimate(res, subp, modelCI)
set.seed(101)
nperm <- 2500
res <- test(res, nperm)
# summary(res)
print(res, estimate = TRUE, cov = FALSE, test = TRUE)

plot(res, ylabel = "4-Year Cumulative Incidence of BCR (%)",
  xlabel = "Median Ki-67 in Subpopulations (% immunoreactivity)",
  tlegend = c("Letrozole", "Tamoxifen"), legendy = 30,
  pointwise = FALSE, ylimit = c(0, 30, -30, 30, 0, 2.8),
  lsty = c(2, 1), marker = c(15, 17), ncex = 0.8, at = 8,
  legend_diff = c(1, 2))

# sensitivity analysis (not reported in the paper)
mineventspop <- c(8, 10, 12)
eventspop <- c(15, 20, 25)
res <- list()
set.seed(101)
nperm <- 2500
cnt <- 1
nm_pv <- numeric(length(mineventspop)*length(eventspop))
for (i in 1:length(mineventspop)) {
  for (j in 1:length(eventspop)) {
    swin <- new("stwin", type = "sliding_events", e1 = mineventspop[i],
      e2 = eventspop[j])
    subp <- new("stsubpop")
    subp <- generate(subp, win = swin, covariate = bigCI$ki67,
      coltrt = bigCI$trt, trts = c(1, 2), coltype = bigCI$event)
    
    res_tmp <- new("steppes")
    modelCI <- new("stmodelCI", coltrt = bigCI$trt, coltime = bigCI$time,
      coltype = bigCI$event, trts = c(1, 2), timePoint = 4)

    res_tmp <- estimate(res_tmp, subp, modelCI)
    res_tmp <- test(res_tmp, nperm)
    nm_pv[cnt] <- paste0("e1_", mineventspop[i], " - e2_", eventspop[j])
    res[[nm_pv[cnt]]] <- res_tmp
    cnt <- cnt + 1
  }
}

sup_pv <- chi2_pv <- HR_pv <- numeric(length(mineventspop)*length(eventspop))
for (i in 1:length(sup_pv)) {
  sup_pv[i] <- res[[cnt]]@result$Res[[1]]$pvalue
  chi2_pv[i] <- res[[cnt]]@result$Res[[1]]$chi2pvalue
  HR_pv[i] <- res[[cnt]]@result$Res[[1]]$HRpvalue
}
all_pv <- cbind(sup_pv = sup_pv, chi2_pv = chi2_pv, HR_pv = HR_pv)
rownames(all_pv) <- nm_pv
all_pv

## Supplementary Material - Efficacy of aspirin in preventing colorectal adenomas
data("aspirin", package = "stepp")

# remove missing values
aspirin_c <- aspirin[complete.cases(aspirin), ]

# extract the subset of patients randomized to either
# placedo or 81 mg/day
aspirin_sub <- subset(aspirin_c, DOSE != 325)

# create outcome indicator
aspirin_sub$Y <- as.numeric(aspirin_sub$AD == 1 | aspirin_sub$AL == 1)

# create treatment indicator
aspirin_sub$trt <- ifelse(aspirin_sub$DOSE == 0, 0, 1)

# STEPP analysis
swin <- new("stwin", type = "sliding", r1 = 30, r2 = 100)
subp <- new("stsubpop")
subp <- generate(subp, win = swin, cov = aspirin_sub$AGE)
summary(subp)

res <- new("steppes")
modelGLM <- new("stmodelGLM", coltrt = aspirin_sub$trt,
  colY = aspirin_sub$Y, glm = "binomial",
  trts = sort(unique(aspirin_sub$trt)))

res <- estimate(res, subp, modelGLM)
set.seed(101)
nperm <- 2500
res <- test(res, nperm)
print(res, estimate = TRUE, cov = FALSE, test = TRUE)

plot(res, ylabel = "Risk of Colorectal Adenomas",
  xlabel = "Median Age in Subpopulations",
  tlegend = c("Placebo", "81 mg/day aspirin"), legendy = 0.65,
  pointwise = FALSE, ylimit = c(0, 0.7, -0.3, 0.6, 0, 3),
  lsty = c(1, 2), marker = c(15, 17), ncex = 0.8, at = 47,
  legend_diff = c(1, 2))
